# cmdline: -v -v -v
# test printing of the parse-tree

for i in ():
    pass
a = None
b = "str"
c = "a very long str that will not be interned"
d = b"bytes"
e = b"a very long bytes that will not be interned"
f = 123456789012345678901234567890
g = 123
